@@ -85,6 +85,8 @@ class audio_low_latency_play(item):
         self.pause_resume = self.var.pause_resume
         self.stop = self.var.stop
         self.ram_cache = self.var.ram_cache
+        self.play_continue = 1
+        self.play_execute_pause = 0
 
 
     def prepare(self):
@@ -246,34 +248,32 @@ class audio_low_latency_play(item):
 
         while len(data) > 0:
 
-            # check duration
-            if self.duration_check:
-                if self.clock.time() - start_time >= self.duration:
-                    break
-
-            # check for stop/pause/resume key
-            if self.pause_resume != u'' or self.stop != u'':
-                key1, time1 = self.kb.get_key()
-                if key1 in self._allowed_responses_stop:
-                    self.kb.flush()
-                    self.show_message(u'Stopped audio playback')
-                    break
-                elif key1 in self._allowed_responses_pause_resume:
-                    self.kb.flush()
-                    self.show_message(u'Paused audio playback')
-                    while True:
-                        key2, time2 = self.kb.get_key()
-                        if key2 in self._allowed_responses_pause_resume:
-                            self.kb.flush()
-                            self.show_message(u'Resumed audio playback')
-                            break
-
             # write data to device
             stream.write(data)
 
             # Read data from wave
             data = wav_file.readframes(chunk)
 
+            # check for stop/pause/resume key
+            if self.pause_resume != u'' or self.stop != u'':
+                self.check_keys()
+
+            while self.play_execute_pause == 1 and self.play_continue == 1:
+                if self.pause_resume != u'' or self.stop != u'':
+                    self.check_keys()
+
+            if self.play_continue == 0:
+                break
+            elif self.duration_check:
+                if self.clock.time() - start_time >= self.duration:
+                    break
+
+            if self.play_continue == 0:
+                break
+            elif self.duration_check:
+                if self.clock.time() - start_time >= self.duration:
+                    break
+
         if self.module == self.experiment.sounddevice_module_name:
             stream.stop()
         elif self.module == self.experiment.pyaudio_module_name:
@@ -301,26 +301,29 @@ class audio_low_latency_play(item):
         start_time = self.clock.time()
 
         for start in range(0,len(wav_data),chunk):
+
+            # write data to device
             stream.write(wav_data[start:start+chunk])
 
-            if self.duration_check:
+            # check for stop/pause/resume key
+            if self.pause_resume != u'' or self.stop != u'':
+                self.check_keys()
+
+            while self.play_execute_pause == 1 and self.play_continue == 1:
+                if self.pause_resume != u'' or self.stop != u'':
+                    self.check_keys()
+
+            if self.play_continue == 0:
+                break
+            elif self.duration_check:
                 if self.clock.time() - start_time >= self.duration:
                     break
-            if self.pause_resume != u'' or self.stop != u'':
-                key1, time1 = self.kb.get_key()
-                if key1 in self._allowed_responses_stop:
-                    self.kb.flush()
-                    self.show_message(u'Stopped audio playback')
+
+            if self.play_continue == 0:
+                break
+            elif self.duration_check:
+                if self.clock.time() - start_time >= self.duration:
                     break
-                elif key1 in self._allowed_responses_pause_resume:
-                    self.kb.flush()
-                    self.show_message(u'Paused audio playback')
-                    while True:
-                        key2, time2 = self.kb.get_key()
-                        if key2 in self._allowed_responses_pause_resume:
-                            self.kb.flush()
-                            self.show_message(u'Resumed audio playback')
-                            break
 
         if self.module == self.experiment.sounddevice_module_name:
             stream.stop()
@@ -332,6 +335,28 @@ class audio_low_latency_play(item):
         self.show_message(u'Stopped audio')
 
 
+    def check_keys(self):
+        """
+        desc:
+            Show message.
+        """
+
+        key1, time1 = self.kb.get_key()
+        self.kb.flush()
+        if self.stop != u'':
+            if key1 in self._allowed_responses_stop:
+                self.show_message(u'Stopped audio playback')
+                self.play_continue = 0
+        if self.pause_resume != u'':
+            if key1 in self._allowed_responses_pause_resume:
+                if self.play_execute_pause == 0:
+                    self.show_message(u'Paused audio playback')
+                    self.play_execute_pause = 1
+                elif self.play_execute_pause == 1:
+                    self.show_message(u'Resumed audio playback')
+                    self.play_execute_pause = 0
+
+
     def show_message(self, message):
         """
         desc:
